@@ -151,6 +151,10 @@ estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('sou
       for (i in 1:length(covariates))
       {
         covariate <- covariates[i]
+        if(!covariate %in% colnames(mcmc$meta$raw_data.original)){
+            warning("Covariate ", covariate, " not available in the data.")
+            next
+        }
         assign(paste0('covariate_', i), factor(as.character(mcmc$meta$raw_data.original[index.by.country, covariate])))
         if (length(levels(get(paste0('covariate_', i)))) > 1)
           regressor <- paste0(regressor, ' + covariate_', i)
@@ -161,6 +165,10 @@ estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('sou
       for (i in 1:length(cont_covariates))
       {
         covariate <- cont_covariates[i]
+        if(!covariate %in% colnames(mcmc$meta$raw_data.original)){
+            warning("Cont. covariate ", covariate, " not available in the data.")
+            next
+        }
         assign(paste0('cont_covariate_', i), mcmc$meta$raw_data.original[index.by.country, covariate])
         if (max(get(paste0('cont_covariate_', i))) - min(get(paste0('cont_covariate_', i))) > 1e-6)
         {
@@ -208,13 +216,15 @@ estimate.bias.sd.original <- function(mcmc, iso.unbiased=NULL, covariates=c('sou
 get.eps.all.phases <- function(Dlpar, mcmc, country)
 {
   eps_return <- numeric(length = mcmc$meta$T_end-1)
-  id2 <- mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country] - 1)
-  dl <- - Dlpar[5]/(1 + exp(- 2*log(9)/Dlpar[1] *(mcmc$meta$tfr_all[id2, country] - Dlpar[1] - Dlpar[2] -Dlpar[3] - Dlpar[4] + 0.5*Dlpar[1]))) + 
-          Dlpar[5]/(1 + exp(- 2*log(9)/Dlpar[3] *(mcmc$meta$tfr_all[id2, country] - Dlpar[4] - 0.5*Dlpar[3])))
-  dl <- dl * ifelse(mcmc$meta$annual.simulation, 1, 5)
-  dl[mcmc$meta$tfr_all[id2, country] < 1] <- 0
-  dl[dl<0] <- 0
-  eps_return[id2] <- mcmc$meta$tfr_all[id2 + 1, country] - mcmc$meta$tfr_all[id2, country] + dl
+  if (mcmc$meta$lambda_c[country] > mcmc$meta$start_c[country]){
+    id2 <- mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country] - 1)
+    dl <- - Dlpar[5]/(1 + exp(- 2*log(9)/Dlpar[1] *(mcmc$meta$tfr_all[id2, country] - Dlpar[1] - Dlpar[2] -Dlpar[3] - Dlpar[4] + 0.5*Dlpar[1]))) + 
+      Dlpar[5]/(1 + exp(- 2*log(9)/Dlpar[3] *(mcmc$meta$tfr_all[id2, country] - Dlpar[4] - 0.5*Dlpar[3])))
+    dl <- dl * ifelse(mcmc$meta$annual.simulation, 1, 5)
+    dl[mcmc$meta$tfr_all[id2, country] < 1] <- 0
+    dl[dl<0] <- 0
+    eps_return[id2] <- mcmc$meta$tfr_all[id2 + 1, country] - mcmc$meta$tfr_all[id2, country] + dl
+  }
   if (mcmc$meta$start_c[country] > 1)
   {
     eps_return[1:(mcmc$meta$start_c[country] - 1)] <- mcmc$meta$tfr_all[2:mcmc$meta$start_c[country], country] - 
@@ -250,7 +260,12 @@ mcmc.update.tfr <- function(country, mcmc)
                sum(exp(mcmc$gamma_ci[country,])), 
              mcmc$Triangle_c4[country], 
              mcmc$d_c[country])
-  epsT.idx <- mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country]-1)
+  if (mcmc$meta$lambda_c[country] > mcmc$meta$start_c[country]){
+    epsT.idx <- mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country]-1)
+  }
+  else{
+    epsT.idx <- c()
+  }
   eps_tfr_prev <- get.eps.all.phases(Dlpar, mcmc, country)
   sd_tfr_prev <- get.sd.all.phases(mcmc, country)
   eps_tfr_prop <- eps_tfr_prev
@@ -357,8 +372,9 @@ mcmc.update.tfr <- function(country, mcmc)
                                                   (mcmc$meta$tfr_all[year, country] - mu) ** 2)/mcmc$finished.iter)
     
   }
-  if (country %in% mcmc$meta3$id_phase3) mcmc$observations[[id3]] <- mcmc$meta$tfr_all[mcmc$meta$lambda_c[country]:mcmc$meta$T_end, country]
-  mcmc$data.list[[country]][epsT.idx] <- mcmc$meta$tfr_all[epsT.idx, country]
+  if (country %in% mcmc$meta3$id_phase3) mcmc$observations[[id3]] <- mcmc$meta$tfr_all[mcmc$meta$lambda_c[country]:mcmc$meta$T_end_c[country], country]
+  if (country %in% mcmc$meta$id_DL)
+    mcmc$data.list[[country]][epsT.idx] <- mcmc$meta$tfr_all[epsT.idx, country]
 }
 
 # get.log.lik.year <- function(year.ind, mcmc, Dlpar, phase3par, id_phase1, id_phase2, id_phase3, tfr=NULL, prev=TRUE)
@@ -570,6 +586,20 @@ mcmc.update.tfr.year <- function(mcmc, countries = NULL)
     id_phase3_r <- intersect(id_phase3_r, countries)
     id_phase2_r <- intersect(id_phase2_r, countries)
   }
+  indices_outliers_m1 <- mcmc$meta$yearly.outliers[[as.character(1)]]
+  if (!is.null(mcmc$meta$ar.phase2) && mcmc$meta$ar.phase2)
+  {
+    indices_outliers_0 <- sort(unique(c(mcmc$meta$yearly.outliers[[as.character(1)]], 
+                                        mcmc$meta$yearly.outliers[[as.character(2)]])))
+    
+    indices_outliers_p1 <- sort(unique(c(mcmc$meta$yearly.outliers[[as.character(1)]], 
+                                         mcmc$meta$yearly.outliers[[as.character(2)]])))
+  }
+  else
+  {
+    indices_outliers_0 <- mcmc$meta$yearly.outliers[[as.character(1)]]
+    indices_outliers_p1 <- mcmc$meta$yearly.outliers[[as.character(2)]]
+  }
   for (year in 1:mcmc$meta$T_end)
   {
     if (year > 1)
@@ -604,26 +634,39 @@ mcmc.update.tfr.year <- function(mcmc, countries = NULL)
     }
     
     ## Compute Log likelihood
+    ## For outliers keep log likelihood to be frozen to 0 for both 
+    ## loglik_orig annd loglik_proposed
     if ((year < mcmc$meta$T_end - 1) && !is.null(mcmc$meta$ar.phase2) && mcmc$meta$ar.phase2)
     {
       loglik_next <- get.log.lik.year(year + 1, mcmc, Dlpar, phase3par)$log.lik
-      loglik_orig <- loglik_orig + loglik_next
       loglik_next_prop <- get.log.lik.year(year + 1, mcmc, Dlpar, phase3par, tfr_proposed, prev_two = TRUE)
+      
+      loglik_next[indices_outliers_p1] <- 0.0
+      loglik_next_prop$log.lik[indices_outliers_p1] <- 0.0
+      loglik_orig <- loglik_orig + loglik_next
       loglik_proposed <- loglik_proposed + loglik_next_prop$log.lik
     }
     if (year < mcmc$meta$T_end)
     {
       loglik_mid <- get.log.lik.year(year, mcmc, Dlpar, phase3par)$log.lik
-      loglik_orig <- loglik_orig + loglik_mid
       loglik_mid_prop <- get.log.lik.year(year, mcmc, Dlpar, phase3par, tfr_proposed)
+      loglik_mid[indices_outliers_0] <- 0.0
+      loglik_mid_prop$log.lik[indices_outliers_0] <- 0.0
+      
+      loglik_orig <- loglik_orig + loglik_mid
+      
       loglik_proposed <- loglik_proposed + loglik_mid_prop$log.lik
     }
     if (year > 1)
     {
-      loglik_orig <- loglik_orig + loglik_prev
       loglik_prev_prop <- get.log.lik.year(year-1, mcmc, Dlpar, phase3par, tfr_proposed, prev = FALSE)
+      loglik_prev[indices_outliers_m1] <- 0.0
+      loglik_prev_prop$log.lik[indices_outliers_m1] <- 0.0
+      
+      loglik_orig <- loglik_orig + loglik_prev
       loglik_proposed <- loglik_proposed + loglik_prev_prop$log.lik
     }
+    # Here should be fine.
     loglik_orig <- loglik_orig + get.log.lik.raw(year, mcmc, mcmc$meta$tfr_all[year, ])
     loglik_proposed <- loglik_proposed + get.log.lik.raw(year, mcmc, tfr_proposed)
     
@@ -665,18 +708,30 @@ mcmc.update.tfr.year <- function(mcmc, countries = NULL)
                                             mcmc$finished.iter * (mu0 - mu) ** 2 + 
                                             (mcmc$meta$tfr_all[year, ] - mu) ** 2)/mcmc$finished.iter)
     
+    indices_outliers_m1 <- indices_outliers_0
+    indices_outliers_0 <- indices_outliers_p1
+    if (!is.null(mcmc$meta$ar.phase2) && mcmc$meta$ar.phase2)
+    {
+      indices_outliers_p1 <- sort(unique(c(mcmc$meta$yearly.outliers[[as.character(year + 1)]], 
+                                           mcmc$meta$yearly.outliers[[as.character(year + 2)]])))
+    }
+    else
+    {
+      indices_outliers_p1 <- mcmc$meta$yearly.outliers[[as.character(year + 2)]]
+    }
   }
   for (country in 1:nr_countries)
   {
     if (!is.null(countries) && !(country %in% countries)) next
-    idx <- mcmc$meta$lambda_c[country]:(mcmc$meta$T_end-1)
     if (country %in% mcmc$meta3$id_phase3) 
     {
       id3 <- which(mcmc$meta$id_phase3 == country)  
-      mcmc$observations[[id3]] <- mcmc$meta$tfr_all[c(idx, mcmc$meta$T_end), country]
+      mcmc$observations[[id3]] <- mcmc$meta$tfr_all[mcmc$meta$lambda_c[country]:mcmc$meta$T_end_c[country], country]
+    }
+    if (country %in% mcmc$meta$id_DL){
+        idx2 <- mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country] - 1)
+        mcmc$data.list[[country]][idx2] <- mcmc$meta$tfr_all[idx2, country]
     }
-    idx2 <- mcmc$meta$start_c[country]:(mcmc$meta$lambda_c[country] - 1)
-    mcmc$data.list[[country]][idx2] <- mcmc$meta$tfr_all[idx2, country]
   }
 }
 
